@@ -13,7 +13,7 @@ import traceback
 import uuid
 from datetime import datetime
 from threading import BoundedSemaphore
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Type
 
 from flask import Flask, Response, jsonify, request
 from six.moves import cStringIO as StringIO
@@ -39,6 +39,7 @@ from localstack.services.awslambda.lambda_utils import (
 )
 from localstack.services.generic_proxy import RegionBackend
 from localstack.services.install import install_go_lambda_runtime
+from localstack.utils import bootstrap
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_models import CodeSigningConfig, LambdaFunction
@@ -48,7 +49,6 @@ from localstack.utils.common import (
     empty_context_manager,
     ensure_readable,
     first_char_to_lower,
-    get_all_subclasses,
     is_zip_file,
     isoformat_milliseconds,
     json_safe,
@@ -70,6 +70,7 @@ from localstack.utils.common import (
     unzip,
 )
 from localstack.utils.docker_utils import DOCKER_CLIENT
+from localstack.utils.generic.singleton_utils import SubtypesInstanceManager
 from localstack.utils.http_utils import canonicalize_headers, parse_chunked_data
 from localstack.utils.run import FuncThread
 
@@ -89,7 +90,7 @@ LAMBDA_JAR_FILE_NAME = "original_lambda_archive.jar"
 LAMBDA_DEFAULT_TIMEOUT = 3
 
 INVALID_PARAMETER_VALUE_EXCEPTION = "InvalidParameterValueException"
-VERSION_LATEST = "$LATEST"
+VERSION_LATEST = LambdaFunction.QUALIFIER_LATEST
 FUNCTION_MAX_SIZE = 69905067
 
 BATCH_SIZE_RANGES = {
@@ -151,17 +152,9 @@ class LambdaRegion(RegionBackend):
         self.event_source_mappings = []
 
 
-class EventSourceListener:
+class EventSourceListener(SubtypesInstanceManager):
     INSTANCES: Dict[str, "EventSourceListener"] = {}
 
-    @classmethod
-    def get(cls, source_type):
-        # TODO: potentially to be replaced with new plugin loading mechanism...
-        if not cls.INSTANCES:
-            for clazz in get_all_subclasses(EventSourceListener):
-                cls.INSTANCES[clazz.source_type()] = clazz()
-        return cls.INSTANCES.get(source_type)
-
     @staticmethod
     def source_type() -> str:
         """Type discriminator - to be implemented by subclasses."""
@@ -186,14 +179,14 @@ class EventSourceListener:
             )
             if self_managed_endpoints.get("KAFKA_BOOTSTRAP_SERVERS"):
                 service_type = "kafka"
-        instance = EventSourceListener.get(service_type)
+        instance = EventSourceListener.get(service_type, raise_if_missing=False)
         if instance:
             instance.start()
 
     @staticmethod
     def process_event_via_listener(service_type: str, event: Any):
         """Process event for the given service type (for reactive mode)"""
-        instance = EventSourceListener.get(service_type)
+        instance = EventSourceListener.get(service_type, raise_if_missing=False)
         if not instance:
             return
 
@@ -203,6 +196,14 @@ class EventSourceListener:
         # start processing in background
         start_worker_thread(_process)
 
+    @classmethod
+    def impl_name(cls) -> str:
+        return cls.source_type()
+
+    @classmethod
+    def get_base_type(cls) -> Type:
+        return EventSourceListener
+
 
 class EventSourceListenerSQS(EventSourceListener):
     # SQS listener thread settings
@@ -244,8 +245,10 @@ class EventSourceListenerSQS(EventSourceListener):
                 return False
 
             LOG.debug(
-                "Found %s source mappings for event from SQS queue %s: %s"
-                % (len(arns), queue_arn, arns)
+                "Found %s source mappings for event from SQS queue %s: %s",
+                len(arns),
+                queue_arn,
+                arns,
             )
             # TODO: support message BatchSize here, same as for polling mode below
             messages = event["Messages"]
@@ -293,9 +296,7 @@ class EventSourceListenerSQS(EventSourceListener):
                     except Exception as e:
                         if "NonExistentQueue" not in str(e):
                             # TODO: remove event source if queue does no longer exist?
-                            LOG.debug(
-                                "Unable to poll SQS messages for queue %s: %s" % (queue_arn, e)
-                            )
+                            LOG.debug("Unable to poll SQS messages for queue %s: %s", queue_arn, e)
 
             except Exception:
                 pass
@@ -307,7 +308,7 @@ class EventSourceListenerSQS(EventSourceListener):
         queue_arn = source["EventSourceArn"]
         region_name = queue_arn.split(":")[3]
         queue_url = aws_stack.sqs_queue_url_for_arn(queue_arn)
-        LOG.debug("Sending event from event source %s to Lambda %s" % (queue_arn, lambda_arn))
+        LOG.debug("Sending event from event source %s to Lambda %s", queue_arn, lambda_arn)
         res = self._send_event_to_lambda(
             queue_arn,
             queue_url,
@@ -508,8 +509,8 @@ def use_docker():
                         "Lambda executor configured as LAMBDA_EXECUTOR=%s but Docker "
                         "is not accessible. Please make sure to mount the Docker socket "
                         "/var/run/docker.sock into the container."
-                    )
-                    % config.LAMBDA_EXECUTOR
+                    ),
+                    config.LAMBDA_EXECUTOR,
                 )
             DO_USE_DOCKER = has_docker
     return DO_USE_DOCKER
@@ -545,12 +546,20 @@ def process_apigateway_invocation(
     is_base64_encoded=False,
     resource_path=None,
     method=None,
-    path_params={},
+    path_params=None,
     query_string_params=None,
-    stage_variables={},
-    request_context={},
-    event_context={},
+    stage_variables=None,
+    request_context=None,
+    event_context=None,
 ):
+    if path_params is None:
+        path_params = {}
+    if stage_variables is None:
+        stage_variables = {}
+    if request_context is None:
+        request_context = {}
+    if event_context is None:
+        event_context = {}
     try:
         resource_path = resource_path or path
         event = construct_invocation_event(
@@ -564,8 +573,10 @@ def process_apigateway_invocation(
         if stage_variables:
             event["stageVariables"] = stage_variables
         LOG.debug(
-            "Running Lambda function %s from API Gateway invocation: %s %s"
-            % (func_arn, method or "GET", path)
+            "Running Lambda function %s from API Gateway invocation: %s %s",
+            func_arn,
+            method or "GET",
+            path,
         )
         asynchronous = not config.SYNCHRONOUS_API_GATEWAY_EVENTS
         inv_result = run_lambda(
@@ -577,8 +588,7 @@ def process_apigateway_invocation(
         return inv_result.result
     except Exception as e:
         LOG.warning(
-            "Unable to run Lambda function on API Gateway message: %s %s"
-            % (e, traceback.format_exc())
+            "Unable to run Lambda function on API Gateway message: %s %s", e, traceback.format_exc()
         )
 
 
@@ -690,7 +700,7 @@ def process_kinesis_records(records, stream_name):
                 )
     except Exception as e:
         LOG.warning(
-            "Unable to run Lambda function on Kinesis records: %s %s" % (e, traceback.format_exc())
+            "Unable to run Lambda function on Kinesis records: %s %s", e, traceback.format_exc()
         )
 
 
@@ -755,13 +765,15 @@ def do_update_alias(arn, alias, version, description=None):
 def run_lambda(
     func_arn,
     event,
-    context={},
+    context=None,
     version=None,
     suppress_output=False,
     asynchronous=False,
     callback=None,
     lock_discriminator: str = None,
 ) -> InvocationResult:
+    if context is None:
+        context = {}
     region_name = func_arn.split(":")[3]
     region = LambdaRegion.get(region_name)
     if suppress_output:
@@ -774,7 +786,7 @@ def run_lambda(
         func_arn = aws_stack.fix_arn(func_arn)
         lambda_function = region.lambdas.get(func_arn)
         if not lambda_function:
-            LOG.debug("Unable to find details for Lambda %s in region %s" % (func_arn, region_name))
+            LOG.debug("Unable to find details for Lambda %s in region %s", func_arn, region_name)
             result = not_found_error(msg="The resource specified in the request does not exist.")
             return InvocationResult(result)
 
@@ -798,9 +810,7 @@ def run_lambda(
             "errorMessage": str(e),
             "stackTrace": traceback.format_tb(exc_traceback),
         }
-        LOG.info(
-            "Error executing Lambda function %s: %s %s" % (func_arn, e, traceback.format_exc())
-        )
+        LOG.info("Error executing Lambda function %s: %s %s", func_arn, e, traceback.format_exc())
         log_output = e.log_output if isinstance(e, lambda_executors.InvocationException) else ""
         return InvocationResult(Response(json.dumps(response), status=500), log_output)
     finally:
@@ -851,7 +861,7 @@ def exec_lambda_code(script, handler_function="handler", lambda_cwd=None, lambda
                     if key not in pre_sys_modules_keys:
                         sys.modules.pop(key)
         except Exception as e:
-            LOG.error("Unable to exec: %s %s" % (script, traceback.format_exc()))
+            LOG.error("Unable to exec: %s %s", script, traceback.format_exc())
             raise e
         finally:
             if lambda_cwd or lambda_env:
@@ -949,8 +959,7 @@ def set_archive_code(code: Dict, lambda_name: str, zip_file_content: bytes = Non
 
 def set_function_code(lambda_function: LambdaFunction):
     def _set_and_configure():
-        lambda_handler = do_set_function_code(lambda_function)
-        lambda_function.versions.get(VERSION_LATEST)["Function"] = lambda_handler
+        do_set_function_code(lambda_function)
         # initialize function code via plugins
         for plugin in lambda_executors.LambdaExecutorPlugin.get_plugins():
             plugin.init_function_code(lambda_function)
@@ -1006,13 +1015,11 @@ def do_set_function_code(lambda_function: LambdaFunction):
             % lambda_name
         )
 
-    region = LambdaRegion.get()
     lambda_name = lambda_function.name()
     arn = lambda_function.arn()
-    lambda_details = region.lambdas[arn]
-    runtime = get_lambda_runtime(lambda_details)
-    lambda_environment = lambda_details.envvars
-    handler_name = lambda_details.handler = lambda_details.handler or LAMBDA_DEFAULT_HANDLER
+    runtime = get_lambda_runtime(lambda_function)
+    lambda_environment = lambda_function.envvars
+    handler_name = lambda_function.handler = lambda_function.handler or LAMBDA_DEFAULT_HANDLER
     code_passed = lambda_function.code
     is_local_mount = code_passed.get("S3Bucket") == config.BUCKET_MARKER_LOCAL
 
@@ -1035,7 +1042,7 @@ def do_set_function_code(lambda_function: LambdaFunction):
         # directory as part of the classpath. Obtain a Java handler function below.
         try:
             lambda_handler = get_java_handler(
-                zip_file_content, archive_file, lambda_function=lambda_details
+                zip_file_content, archive_file, lambda_function=lambda_function
             )
         except Exception as e:
             # this can happen, e.g., for Lambda code mounted via __local__ -> ignore
@@ -1051,7 +1058,6 @@ def do_set_function_code(lambda_function: LambdaFunction):
     # Obtain handler details for any non-Java Lambda function
     if not is_java:
         handler_file = get_handler_file_from_name(handler_name, runtime=runtime)
-
         main_file = "%s/%s" % (lambda_cwd, handler_file)
 
         if CHECK_HANDLER_ON_CREATION and not os.path.exists(main_file):
@@ -1065,11 +1071,10 @@ def do_set_function_code(lambda_function: LambdaFunction):
                     use_docker(),
                     config.LAMBDA_REMOTE_DOCKER,
                 )
-                LOG.debug("Lambda archive content:\n%s" % file_list)
+                LOG.debug("Lambda archive content:\n%s", file_list)
                 raise ClientError(
                     error_response(
-                        "Unable to find handler script (%s) in Lambda archive. %s"
-                        % (main_file, config_debug),
+                        f"Unable to find handler script ({main_file}) in Lambda archive. {config_debug}",
                         400,
                         error_type="ValidationError",
                     )
@@ -1116,6 +1121,9 @@ def do_set_function_code(lambda_function: LambdaFunction):
 
             lambda_handler = execute_go
 
+    if lambda_handler:
+        lambda_executors.LambdaExecutorLocal.add_function_callable(lambda_function, lambda_handler)
+
     return lambda_handler
 
 
@@ -1731,8 +1739,10 @@ def invoke_function(function):
     invocation_type = request.headers.get("X-Amz-Invocation-Type", "RequestResponse")
     log_type = request.headers.get("X-Amz-Log-Type")
 
-    def _create_response(invocation_result, status_code=200, headers={}):
+    def _create_response(invocation_result, status_code=200, headers=None):
         """Create the final response for the given invocation result."""
+        if headers is None:
+            headers = {}
         if not isinstance(invocation_result, InvocationResult):
             invocation_result = InvocationResult(invocation_result)
         result = invocation_result.result
@@ -1792,7 +1802,7 @@ def invoke_function(function):
             if forward_result is not None:
                 return _create_response(forward_result)
         except Exception as e:
-            LOG.debug('Unable to forward Lambda invocation to fallback URL: "%s" - %s' % (data, e))
+            LOG.debug('Unable to forward Lambda invocation to fallback URL: "%s" - %s', data, e)
         return not_found
 
     if invocation_type == "RequestResponse":
@@ -2124,6 +2134,9 @@ def delete_function_event_invoke_config(function):
     region = LambdaRegion.get()
     try:
         function_arn = func_arn(function)
+        if function_arn not in region.lambdas:
+            msg = f"Function not found: {function_arn}"
+            return not_found_error(msg)
         lambda_obj = region.lambdas[function_arn]
     except Exception as e:
         return error_response(str(e), 400)
@@ -2319,11 +2332,29 @@ def update_code_signing_config(arn):
     return Response(json.dumps(result), status=200)
 
 
+def validate_lambda_config():
+    """Validates important config variables necessary for flawless lambda execution"""
+    if (
+        config.LAMBDA_DOCKER_NETWORK
+        and config.is_in_docker
+        and config.LAMBDA_DOCKER_NETWORK
+        not in DOCKER_CLIENT.get_networks(bootstrap.get_main_container_name())
+    ):
+        LOG.warning(
+            "Your specified LAMBDA_DOCKER_NETWORK '%s' is not connected to the main LocalStack container '%s'. "
+            "Lambda functionality might be severely limited.",
+            config.LAMBDA_DOCKER_NETWORK,
+            bootstrap.get_main_container_name(),
+        )
+
+
 def serve(port):
     from localstack.services import generic_proxy  # moved here to fix circular import errors
 
     # initialize the Lambda executor
     LAMBDA_EXECUTOR.startup()
+    # print warnings for potentially incorrect config options
+    validate_lambda_config()
 
     # initialize/import plugins - TODO find better place to import plugins! (to be integrated into proper plugin model)
     import localstack.contrib.thundra  # noqa
