@@ -19,6 +19,7 @@ from localstack.constants import DEFAULT_LAMBDA_CONTAINER_REGISTRY
 from localstack.services.awslambda.lambda_utils import (
     API_PATH_ROOT,
     LAMBDA_RUNTIME_PROVIDED,
+    get_container_network_for_lambda,
     get_main_endpoint_from_container,
     get_record_from_event,
     is_java_lambda,
@@ -308,7 +309,7 @@ class LambdaInvocationForwarderPlugin(LambdaExecutorPlugin):
         data = run_safe(lambda: to_str(event)) or event
         data = json.dumps(json_safe(data)) if isinstance(data, dict) else str(data)
         LOG.debug(
-            "Forwarding Lambda invocation to LAMBDA_FORWARD_URL: %s" % config.LAMBDA_FORWARD_URL
+            "Forwarding Lambda invocation to LAMBDA_FORWARD_URL: %s", config.LAMBDA_FORWARD_URL
         )
         result = safe_requests.post(url, data, headers=headers)
         if result.status_code >= 400:
@@ -317,8 +318,9 @@ class LambdaInvocationForwarderPlugin(LambdaExecutorPlugin):
             )
         content = run_safe(lambda: to_str(result.content)) or result.content
         LOG.debug(
-            "Received result from external Lambda endpoint (status %s): %s"
-            % (result.status_code, content)
+            "Received result from external Lambda endpoint (status %s): %s",
+            result.status_code,
+            content,
         )
         result = InvocationResult(content)
         return result
@@ -582,8 +584,8 @@ class LambdaExecutorContainers(LambdaExecutor):
         if is_large_event:
             # in case of very large event payloads, we need to pass them via stdin
             LOG.debug(
-                "Received large Lambda event payload (length %s) - passing via stdin"
-                % len(event_body)
+                "Received large Lambda event payload (length %s) - passing via stdin",
+                len(event_body),
             )
             env_vars["DOCKER_LAMBDA_USE_STDIN"] = "1"
 
@@ -679,7 +681,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         # prepare event body
         if not event:
             LOG.info(
-                'Empty event body specified for invocation of Lambda "%s"' % lambda_function.arn()
+                'Empty event body specified for invocation of Lambda "%s"', lambda_function.arn()
             )
             event = {}
         event_body = json.dumps(json_safe(event))
@@ -711,8 +713,8 @@ class LambdaExecutorContainers(LambdaExecutor):
                 )
             else:
                 LOG.debug(
-                    "Passing JVM options to container environment: JAVA_TOOL_OPTIONS=%s"
-                    % config.LAMBDA_JAVA_OPTS
+                    "Passing JVM options to container environment: JAVA_TOOL_OPTIONS=%s",
+                    config.LAMBDA_JAVA_OPTS,
                 )
                 environment["JAVA_TOOL_OPTIONS"] = config.LAMBDA_JAVA_OPTS
 
@@ -721,7 +723,7 @@ class LambdaExecutorContainers(LambdaExecutor):
             environment["NODE_TLS_REJECT_UNAUTHORIZED"] = "0"
 
         # run Lambda executor and fetch invocation result
-        LOG.info("Running lambda: %s" % lambda_function.arn())
+        LOG.info("Running lambda: %s", lambda_function.arn())
         result = self.run_lambda_executor(lambda_function=lambda_function, inv_context=inv_context)
 
         return result
@@ -901,7 +903,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             container_name = self.get_container_name(func_arn)
 
             status = self.get_docker_container_status(func_arn)
-            LOG.debug('Priming Docker container (status "%s"): %s' % (status, container_name))
+            LOG.debug('Priming Docker container (status "%s"): %s', status, container_name)
 
             docker_image = Util.docker_image_for_lambda(lambda_function)
 
@@ -911,12 +913,12 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 self.destroy_docker_container(func_arn)
 
                 # get container startup command and run it
-                LOG.debug("Creating container: %s" % container_name)
+                LOG.debug("Creating container: %s", container_name)
                 self.create_container(lambda_function, env_vars, lambda_cwd, docker_flags)
 
                 if config.LAMBDA_REMOTE_DOCKER:
                     LOG.debug(
-                        'Copying files to container "%s" from "%s".' % (container_name, lambda_cwd)
+                        'Copying files to container "%s" from "%s".', container_name, lambda_cwd
                     )
                     DOCKER_CLIENT.copy_into_container(
                         container_name, "%s/." % lambda_cwd, DOCKER_TASK_FOLDER
@@ -943,8 +945,10 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             entry_point = DOCKER_CLIENT.get_image_entrypoint(docker_image)
 
             LOG.debug(
-                'Using entrypoint "%s" for container "%s" on network "%s".'
-                % (entry_point, container_name, container_network)
+                'Using entrypoint "%s" for container "%s" on network "%s".',
+                entry_point,
+                container_name,
+                container_network,
             )
 
             return ContainerInfo(container_name, entry_point)
@@ -965,7 +969,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         # make sure AWS_LAMBDA_EVENT_BODY is not set (otherwise causes issues with "docker exec ..." above)
         env_vars.pop("AWS_LAMBDA_EVENT_BODY", None)
 
-        network = config.LAMBDA_DOCKER_NETWORK
+        network = get_container_network_for_lambda()
         additional_flags = docker_flags
 
         dns = config.LAMBDA_DOCKER_DNS
@@ -1021,12 +1025,12 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             # Get the container name and id.
             container_name = self.get_container_name(func_arn)
             if status == 1:
-                LOG.debug("Stopping container: %s" % container_name)
+                LOG.debug("Stopping container: %s", container_name)
                 DOCKER_CLIENT.stop_container(container_name)
                 status = self.get_docker_container_status(func_arn)
 
             if status == -1:
-                LOG.debug("Removing container: %s" % container_name)
+                LOG.debug("Removing container: %s", container_name)
                 rm_docker_container(container_name, safe=True)
 
             # clean up function invoke times, as some init logic depends on this
@@ -1054,7 +1058,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         with self.docker_container_lock:
             container_names = self.get_all_container_names()
 
-            LOG.debug("Removing %d containers." % len(container_names))
+            LOG.debug("Removing %d containers.", len(container_names))
             for container_name in container_names:
                 DOCKER_CLIENT.remove_container(container_name)
 
@@ -1089,7 +1093,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             # Get the container name.
             container_name = self.get_container_name(func_arn)
 
-            container_network = DOCKER_CLIENT.get_network(container_name)
+            container_network = DOCKER_CLIENT.get_networks(container_name)[0]
 
             return container_network
 
@@ -1164,7 +1168,7 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
             inv_context.lambda_command = inv_context.handler
 
         # add Docker Lambda env vars
-        network = config.LAMBDA_DOCKER_NETWORK or None
+        network = get_container_network_for_lambda()
         if network == "host":
             port = get_free_tcp_port()
             env_vars["DOCKER_LAMBDA_API_PORT"] = port
@@ -1217,6 +1221,10 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
 
 
 class LambdaExecutorLocal(LambdaExecutor):
+
+    # maps functionARN -> functionVersion -> callable used to invoke a Lambda function locally
+    FUNCTION_CALLABLES: Dict[str, Dict[str, Callable]] = {}
+
     def _execute_in_custom_runtime(
         self, cmd: Union[str, List[str]], lambda_function: LambdaFunction = None
     ) -> InvocationResult:
@@ -1295,7 +1303,9 @@ class LambdaExecutorLocal(LambdaExecutor):
         # execute the Lambda function in a forked sub-process, sync result via queue
         queue = Queue()
 
-        lambda_function_callable = lambda_function.function(inv_context.function_version)
+        lambda_function_callable = self.get_lambda_callable(
+            lambda_function, qualifier=inv_context.function_version
+        )
 
         def do_execute():
             # now we're executing in the child process, safe to change CWD and ENV
@@ -1433,6 +1443,19 @@ class LambdaExecutorLocal(LambdaExecutor):
         result = self._execute_in_custom_runtime(cmd, lambda_function=lambda_function)
         return result
 
+    def execute_go_lambda(self, event, context, main_file, lambda_function: LambdaFunction = None):
+
+        if lambda_function:
+            lambda_function.envvars["AWS_LAMBDA_FUNCTION_HANDLER"] = main_file
+            lambda_function.envvars["AWS_LAMBDA_EVENT_BODY"] = json.dumps(json_safe(event))
+        else:
+            LOG.warning("Unable to get function details for local execution of Golang Lambda")
+
+        cmd = GO_LAMBDA_RUNTIME
+        LOG.debug("Running Golang Lambda with runtime: %s", cmd)
+        result = self._execute_in_custom_runtime(cmd, lambda_function=lambda_function)
+        return result
+
     @staticmethod
     def set_default_env_variables():
         # set default env variables required for most Lambda handlers
@@ -1449,18 +1472,25 @@ class LambdaExecutorLocal(LambdaExecutor):
             if env_value_before is None:
                 os.environ.pop(env_name, None)
 
-    def execute_go_lambda(self, event, context, main_file, lambda_function: LambdaFunction = None):
+    @classmethod
+    def get_lambda_callable(cls, function: LambdaFunction, qualifier: str = None) -> Callable:
+        """Returns the function Callable for invoking the given function locally"""
+        qualifier = function.get_qualifier_version(qualifier)
+        func_dict = cls.FUNCTION_CALLABLES.get(function.arn()) or {}
+        # TODO: function versioning and qualifiers should be refactored and designed properly!
+        callable = func_dict.get(qualifier) or func_dict.get(LambdaFunction.QUALIFIER_LATEST)
+        if not callable:
+            raise Exception(
+                f"Unable to find callable for Lambda function {function.arn()} - {qualifier}"
+            )
+        return callable
 
-        if lambda_function:
-            lambda_function.envvars["AWS_LAMBDA_FUNCTION_HANDLER"] = main_file
-            lambda_function.envvars["AWS_LAMBDA_EVENT_BODY"] = json.dumps(json_safe(event))
-        else:
-            LOG.warning("Unable to get function details for local execution of Golang Lambda")
-
-        cmd = GO_LAMBDA_RUNTIME
-        LOG.info(cmd)
-        result = self._execute_in_custom_runtime(cmd, lambda_function=lambda_function)
-        return result
+    @classmethod
+    def add_function_callable(cls, function: LambdaFunction, lambda_handler: Callable):
+        """Sets the function Callable for invoking the $LATEST version of the Lambda function."""
+        func_dict = cls.FUNCTION_CALLABLES.setdefault(function.arn(), {})
+        qualifier = function.get_qualifier_version(LambdaFunction.QUALIFIER_LATEST)
+        func_dict[qualifier] = lambda_handler
 
 
 class Util:
@@ -1557,9 +1587,7 @@ def log_lambda_result(func_arn, result, log_output):
     result = to_str(result or "")
     log_output = truncate(to_str(log_output or ""), max_length=1000)
     log_formatted = truncate(log_output.strip().replace("\n", "\n> "), max_length=1000)
-    LOG.debug(
-        "Lambda %s result / log output:\n%s\n> %s" % (func_arn, result.strip(), log_formatted)
-    )
+    LOG.debug("Lambda %s result / log output:\n%s\n> %s", func_arn, result.strip(), log_formatted)
 
 
 # --------------
