@@ -2,7 +2,7 @@ import json
 import os
 from typing import Callable
 
-from localstack.constants import INSTALL_DIR_INFRA
+from localstack.config import dirs
 from localstack.utils import common
 
 # URL to "cfn-response" module which is required in some CF Lambdas
@@ -119,7 +119,7 @@ def param_json_to_str(name):
 
 
 def get_cfn_response_mod_file():
-    cfn_response_tmp_file = os.path.join(INSTALL_DIR_INFRA, "lambda.cfn-response.js")
+    cfn_response_tmp_file = os.path.join(dirs.static_libs, "lambda.cfn-response.js")
     if not os.path.exists(cfn_response_tmp_file):
         common.download(CFN_RESPONSE_MODULE_URL, cfn_response_tmp_file)
     return cfn_response_tmp_file
