@@ -7,7 +7,7 @@ import threading
 import time
 import traceback
 import uuid
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Union
 
 import requests
 from boto3.dynamodb.types import TypeDeserializer
@@ -35,6 +35,7 @@ from localstack.utils.common import (
     timestamp,
     to_bytes,
     to_str,
+    truncate,
 )
 from localstack.utils.kinesis import kinesis_connector
 
@@ -86,9 +87,7 @@ def get_delivery_stream_tags(
     start_i = -1
     if exclusive_start_tag_key is not None:
         start_i = next(
-            iter(
-                [i for i, tag in enumerate(stream["Tags"]) if tag["Key"] == exclusive_start_tag_key]
-            )
+            iter(i for i, tag in enumerate(stream["Tags"]) if tag["Key"] == exclusive_start_tag_key)
         )
 
     response["Tags"] = [tag for i, tag in enumerate(stream["Tags"]) if start_i < i < limit]
@@ -117,7 +116,7 @@ def preprocess_records(processor: Dict, records: List[Dict]) -> List[Dict]:
     return records
 
 
-def add_missing_record_attributes(records: List[Dict]):
+def add_missing_record_attributes(records: List[Dict]) -> None:
     def _get_entry(obj, key):
         return obj.get(key) or obj.get(first_char_to_lower(key))
 
@@ -151,7 +150,27 @@ def put_record(stream_name: str, record: Dict) -> Dict:
     return put_records(stream_name, [record])
 
 
-def put_records(stream_name: str, records: List[Dict]) -> Dict:
+def put_records_to_s3_bucket(
+    stream_name: str, records: List[Dict], s3_configuration: Dict[str, Union[str, Dict]]
+):
+    bucket = s3_bucket_name(s3_configuration["BucketARN"])
+    prefix = s3_configuration.get("Prefix", "")
+
+    s3 = connect_to_resource("s3")
+    batched_data = b"".join([base64.b64decode(r.get("Data", r.get("data"))) for r in records])
+
+    obj_path = get_s3_object_path(stream_name, prefix)
+    try:
+        LOG.debug("Publishing to S3 destination: %s. Data: %s", bucket, batched_data)
+        s3.Object(bucket, obj_path).put(Body=batched_data)
+    except Exception as e:
+        LOG.error(
+            "Unable to put records %s to s3 bucket: %s %s", records, e, traceback.format_exc()
+        )
+        raise e
+
+
+def put_records(stream_name: str, unprocessed_records: List[Dict]) -> Dict:
     """Put a list of records to the firehose stream - either directly from a PutRecord API call, or
     received from an underlying Kinesis stream (if 'KinesisStreamAsSource' is configured)"""
     stream = get_stream(stream_name)
@@ -159,7 +178,7 @@ def put_records(stream_name: str, records: List[Dict]) -> Dict:
         return error_not_found(stream_name)
 
     # preprocess records, add any missing attributes
-    add_missing_record_attributes(records)
+    add_missing_record_attributes(unprocessed_records)
 
     for dest in stream.get("Destinations", []):
 
@@ -169,6 +188,7 @@ def put_records(stream_name: str, records: List[Dict]) -> Dict:
             proc_config = (
                 isinstance(child, dict) and child.get("ProcessingConfiguration") or proc_config
             )
+        records = unprocessed_records
         if proc_config.get("Enabled") is not False:
             for processor in proc_config.get("Processors", []):
                 # TODO: run processors asynchronously, to avoid request timeouts on PutRecord API calls
@@ -181,6 +201,20 @@ def put_records(stream_name: str, records: List[Dict]) -> Dict:
             es = connect_elasticsearch(
                 endpoint=es_dest.get("ClusterEndpoint"), domain=es_dest.get("DomainARN")
             )
+            # TODO support FailedDocumentsOnly as well
+            if es_dest.get("S3BackupMode") == "AllDocuments":
+                s3_config = es_dest.get("S3Configuration")
+                if s3_config:
+                    try:
+                        put_records_to_s3_bucket(
+                            stream_name=stream_name,
+                            records=unprocessed_records,
+                            s3_configuration=s3_config,
+                        )
+                    except Exception as e:
+                        LOG.warning("Unable to backup unprocessed records to S3. Error: %s", e)
+                else:
+                    LOG.warning("Passed S3BackupMode without S3Configuration. Cannot backup...")
             for record in records:
                 obj_id = uuid.uuid4()
 
@@ -192,27 +226,21 @@ def put_records(stream_name: str, records: List[Dict]) -> Dict:
                 elif "data" in record:
                     data = base64.b64decode(record["data"])
 
-                body = json.loads(data)
+                try:
+                    body = json.loads(data)
+                except Exception as e:
+                    LOG.warning("Elasticsearch only allows json input data!")
+                    raise e
 
+                LOG.debug("Publishing to ES destination. Data: %s", truncate(data, max_length=300))
                 try:
                     es.create(index=es_index, doc_type=es_type, id=obj_id, body=body)
                 except Exception as e:
-                    LOG.error("Unable to put record to stream: %s %s" % (e, traceback.format_exc()))
+                    LOG.error("Unable to put record to stream: %s %s", e, traceback.format_exc())
                     raise e
         if "S3DestinationDescription" in dest:
-            s3_dest = dest["S3DestinationDescription"]
-            bucket = s3_bucket_name(s3_dest["BucketARN"])
-            prefix = s3_dest.get("Prefix", "")
-
-            s3 = connect_to_resource("s3")
-            batched_data = b"".join([base64.b64decode(r.get("Data") or r["data"]) for r in records])
-
-            obj_path = get_s3_object_path(stream_name, prefix)
-            try:
-                s3.Object(bucket, obj_path).put(Body=batched_data)
-            except Exception as e:
-                LOG.error("Unable to put record to stream: %s %s" % (e, traceback.format_exc()))
-                raise e
+            s3_dest_config = dest["S3DestinationDescription"]
+            put_records_to_s3_bucket(stream_name, records, s3_dest_config)
         if "HttpEndpointDestinationDescription" in dest:
             http_dest = dest["HttpEndpointDestinationDescription"]
             end_point = http_dest["EndpointConfiguration"]
@@ -232,8 +260,10 @@ def put_records(stream_name: str, records: List[Dict]) -> Dict:
                 requests.post(url, json=record_to_send, headers=headers)
             except Exception as e:
                 LOG.info(
-                    "Unable to put Firehose records to HTTP endpoint %s: %s %s"
-                    % (url, e, traceback.format_exc())
+                    "Unable to put Firehose records to HTTP endpoint %s: %s %s",
+                    url,
+                    e,
+                    traceback.format_exc(),
                 )
                 raise e
     return {"RecordId": str(uuid.uuid4())}
@@ -380,7 +410,7 @@ def get_stream(stream_name: str, format_s3_dest: bool = False) -> Optional[Dict]
 
 
 def is_extended_s3_destination(s3_dest: Dict) -> bool:
-    return any([s3_dest.get(attr) is not None for attr in S3_EXTENDED_DEST_ATTRS])
+    return any(s3_dest.get(attr) is not None for attr in S3_EXTENDED_DEST_ATTRS)
 
 
 def error_not_found(stream_name: str):
