@@ -178,8 +178,10 @@ class MotoServerProperties:
 
 
 def start_proxy_for_service(
-    service_name, port, backend_port, update_listener, quiet=False, params={}
+    service_name, port, backend_port, update_listener, quiet=False, params=None
 ):
+    if params is None:
+        params = {}
     # TODO: remove special switch for Elasticsearch (see also note in service_port(...) in config.py)
     if config.FORWARD_EDGE_INMEM and service_name != "elasticsearch":
         if backend_port:
@@ -435,7 +437,7 @@ def do_start_infra(asynchronous, apis, is_in_docker):
         install.install_debugpy_and_dependencies()
         import debugpy
 
-        LOG.info("Starting debug server at: %s:%s" % (constants.BIND_HOST, config.DEVELOP_PORT))
+        LOG.info("Starting debug server at: %s:%s", constants.BIND_HOST, config.DEVELOP_PORT)
         debugpy.listen((constants.BIND_HOST, config.DEVELOP_PORT))
 
         if config.WAIT_FOR_DEBUGGER:
