@@ -34,6 +34,7 @@ from localstack.runtime import hooks
 from localstack.utils.common import (
     chmod_r,
     download,
+    file_exists_not_empty,
     get_arch,
     get_os,
     is_windows,
@@ -75,11 +76,22 @@ URL_LOCALSTACK_FAT_JAR = (
 MARKER_FILE_LIGHT_VERSION = "%s/.light-version" % dirs.static_libs
 IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local"
 ARTIFACTS_REPO = "https://github.com/localstack/localstack-artifacts"
+SFN_PATCH_URL_PREFIX = (
+    f"{ARTIFACTS_REPO}/raw/047cc6dcd2e31f5ff3ec52d293c61b875f606958/stepfunctions-local-patch"
+)
 SFN_PATCH_CLASS1 = "com/amazonaws/stepfunctions/local/runtime/Config.class"
 SFN_PATCH_CLASS2 = (
     "com/amazonaws/stepfunctions/local/runtime/executors/task/LambdaTaskStateExecutor.class"
 )
-SFN_PATCH_URL_PREFIX = f"{ARTIFACTS_REPO}/raw/master/stepfunctions-local-patch"
+SFN_PATCH_CLASS_STARTER = "cloud/localstack/StepFunctionsStarter.class"
+SFN_PATCH_CLASS_REGION = "cloud/localstack/RegionAspect.class"
+SFN_PATCH_FILE_METAINF = "META-INF/aop.xml"
+
+# additional JAR libs required for multi-region and persistence (PRO only) support
+MAVEN_REPO = "https://repo1.maven.org/maven2"
+URL_ASPECTJRT = f"{MAVEN_REPO}/org/aspectj/aspectjrt/1.9.7/aspectjrt-1.9.7.jar"
+URL_ASPECTJWEAVER = f"{MAVEN_REPO}/org/aspectj/aspectjweaver/1.9.7/aspectjweaver-1.9.7.jar"
+JAR_URLS = [URL_ASPECTJRT, URL_ASPECTJWEAVER]
 
 # kinesis-mock version
 KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.2.0"
@@ -173,7 +185,7 @@ def install_elasticsearch(version=None):
             plugin_binary = os.path.join(install_dir, "bin", "elasticsearch-plugin")
             plugin_dir = os.path.join(install_dir, "plugins", plugin)
             if not os.path.exists(plugin_dir):
-                LOG.info("Installing Elasticsearch plugin %s" % plugin)
+                LOG.info("Installing Elasticsearch plugin %s", plugin)
 
                 def try_install():
                     safe_run([plugin_binary, "install", "-b", plugin])
@@ -337,11 +349,39 @@ def install_stepfunctions_local():
         for file in path.glob("*.jar"):
             file.rename(Path(INSTALL_DIR_STEPFUNCTIONS) / file.name)
         rm_rf("%s/stepfunctionslocal" % dirs.static_libs)
-    # apply patches
-    for patch_class in (SFN_PATCH_CLASS1, SFN_PATCH_CLASS2):
+
+    classes = [
+        SFN_PATCH_CLASS1,
+        SFN_PATCH_CLASS2,
+        SFN_PATCH_CLASS_REGION,
+        SFN_PATCH_CLASS_STARTER,
+        SFN_PATCH_FILE_METAINF,
+    ]
+    for patch_class in classes:
         patch_url = f"{SFN_PATCH_URL_PREFIX}/{patch_class}"
         add_file_to_jar(patch_class, patch_url, target_jar=INSTALL_PATH_STEPFUNCTIONS_JAR)
 
+    # special case for Manifest file - extract first, replace content, then update in JAR file
+    manifest_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, "META-INF", "MANIFEST.MF")
+    if not os.path.exists(manifest_file):
+        content = run(["unzip", "-p", INSTALL_PATH_STEPFUNCTIONS_JAR, "META-INF/MANIFEST.MF"])
+        content = re.sub(
+            "Main-Class: .+", "Main-Class: cloud.localstack.StepFunctionsStarter", content
+        )
+        classpath = " ".join([os.path.basename(jar) for jar in JAR_URLS])
+        content = re.sub(r"Class-Path: \. ", f"Class-Path: {classpath} . ", content)
+        save_file(manifest_file, content)
+        run(
+            ["zip", INSTALL_PATH_STEPFUNCTIONS_JAR, "META-INF/MANIFEST.MF"],
+            cwd=INSTALL_DIR_STEPFUNCTIONS,
+        )
+
+    # download additional jar libs
+    for jar_url in JAR_URLS:
+        target = os.path.join(INSTALL_DIR_STEPFUNCTIONS, os.path.basename(jar_url))
+        if not file_exists_not_empty(target):
+            download(jar_url, target)
+
 
 def add_file_to_jar(class_file, class_url, target_jar, base_dir=None):
     base_dir = base_dir or os.path.dirname(target_jar)
@@ -512,7 +552,7 @@ def install_debugpy_and_dependencies():
 
 def log_install_msg(component, verbatim=False):
     component = component if verbatim else "local %s server" % component
-    LOG.info("Downloading and installing %s. This may take some time." % component)
+    LOG.info("Downloading and installing %s. This may take some time.", component)
 
 
 def download_and_extract(archive_url, target_dir, retries=0, sleep=3, tmp_archive=None):
@@ -547,7 +587,7 @@ def download_and_extract_with_retry(archive_url, tmp_archive, target_dir):
         download_and_extract(archive_url, target_dir, tmp_archive=tmp_archive)
     except Exception as e:
         # try deleting and re-downloading the zip file
-        LOG.info("Unable to extract file, re-downloading ZIP archive %s: %s" % (tmp_archive, e))
+        LOG.info("Unable to extract file, re-downloading ZIP archive %s: %s", tmp_archive, e)
         rm_rf(tmp_archive)
         download_and_extract(archive_url, target_dir, tmp_archive=tmp_archive)
 
@@ -600,7 +640,7 @@ class InstallerManager:
 
     @functools.lru_cache()
     def get_installers(self) -> Dict[str, Callable]:
-        installer: List[Installer] = list()
+        installer: List[Installer] = []
 
         for repo in self.repositories.load_all():
             installer.extend(repo.get_installer())
