@@ -7,6 +7,8 @@ import logging
 import time
 import traceback
 import uuid
+from typing import Dict, List
+from urllib.parse import parse_qs, urlparse
 
 import requests
 import six
@@ -15,7 +17,6 @@ from flask import Response as FlaskResponse
 from moto.sns.exceptions import DuplicateSnsEndpointError
 from moto.sns.models import SNSBackend as MotoSNSBackend
 from requests.models import Request, Response
-from six.moves.urllib import parse as urlparse
 
 from localstack.config import external_service_url
 from localstack.constants import MOTO_ACCOUNT_ID, TEST_AWS_ACCOUNT_ID
@@ -47,25 +48,32 @@ LOG = logging.getLogger(__name__)
 # additional attributes used for HTTP subscriptions
 HTTP_SUBSCRIPTION_ATTRIBUTES = ["UnsubscribeURL"]
 
+# actions to be skipped from persistence
+SKIP_PERSISTENCE_ACTIONS = [
+    "Subscribe",
+    "ConfirmSubscription",
+    "Unsubscribe",
+]
+
 
 class SNSBackend(RegionBackend):
+    # maps topic ARN to list of subscriptions
+    sns_subscriptions: Dict[str, List[Dict]]
+    # maps subscription ARN to subscription status
+    subscription_status: Dict[str, Dict]
+    # maps topic ARN to list of tags
+    sns_tags: Dict[str, List[Dict]]
+    # cache of topic ARN to platform endpoint messages (used primarily for testing)
+    platform_endpoint_messages: Dict[str, List[Dict]]
+    # list of sent SMS messages - TODO: expose via internal API
+    sms_messages: List[Dict]
+
     def __init__(self):
-        # maps topic ARN to list of subscriptions
         self.sns_subscriptions = {}
-        # maps subscription ARN to subscription status
         self.subscription_status = {}
-        # maps topic ARN to list of tags
         self.sns_tags = {}
-        # cache of topic ARN to platform endpoint messages (used primarily for testing)
         self.platform_endpoint_messages = {}
-        # maps phone numbers to list of sent messages
         self.sms_messages = []
-        # actions to be skipped from persistence
-        self.skip_persistence_actions = [
-            "Subscribe",
-            "ConfirmSubscription",
-            "Unsubscribe",
-        ]
 
 
 class ProxyListenerSNS(PersistingProxyListener):
@@ -85,12 +93,12 @@ class ProxyListenerSNS(PersistingProxyListener):
 
         if method == "POST":
             # parse payload and extract fields
-            req_data = urlparse.parse_qs(to_str(data), keep_blank_values=True)
+            req_data = parse_qs(to_str(data), keep_blank_values=True)
 
             # parse data from query path
             if not req_data:
-                parsed_path = urlparse.urlparse(path)
-                req_data = urlparse.parse_qs(parsed_path.query, keep_blank_values=True)
+                parsed_path = urlparse(path)
+                req_data = parse_qs(parsed_path.query, keep_blank_values=True)
 
             req_action = req_data["Action"][0]
             topic_arn = (
@@ -135,6 +143,13 @@ class ProxyListenerSNS(PersistingProxyListener):
                 if "Endpoint" not in req_data:
                     return make_error(message="Endpoint not specified in subscription", code=400)
 
+                if ".fifo" in req_data["Endpoint"][0] and ".fifo" not in topic_arn:
+                    return make_error(
+                        message="FIFO SQS Queues can not be subscribed to standard SNS topics",
+                        code=400,
+                        code_string="InvalidParameter",
+                    )
+
             elif req_action == "ConfirmSubscription":
                 if "TopicArn" not in req_data:
                     return make_error(
@@ -171,6 +186,14 @@ class ProxyListenerSNS(PersistingProxyListener):
                     return make_error(
                         code=400, code_string="InvalidParameter", message="Empty message"
                     )
+
+                if topic_arn and ".fifo" in topic_arn and not req_data.get("MessageGroupId"):
+                    return make_error(
+                        code=400,
+                        code_string="InvalidParameter",
+                        message="The MessageGroupId parameter is required for FIFO topics",
+                    )
+
                 sns_backend = SNSBackend.get()
                 # No need to create a topic to send SMS or single push notifications with SNS
                 # but we can't mock a sending so we only return that it went well
@@ -278,7 +301,7 @@ class ProxyListenerSNS(PersistingProxyListener):
             response_regex_replace(response, search, "")
 
             # parse request and extract data
-            req_data = urlparse.parse_qs(to_str(data))
+            req_data = parse_qs(to_str(data))
             req_action = req_data["Action"][0]
             if req_action == "Subscribe" and response.status_code < 400:
                 response_data = xmltodict.parse(response.content)
@@ -311,10 +334,9 @@ class ProxyListenerSNS(PersistingProxyListener):
                 )
 
     def should_persist(self, method, path, data, headers, response):
-        sns_backend = SNSBackend.get()
         req_params = parse_request_data(method, path, data)
         action = req_params.get("Action", "")
-        if action in sns_backend.skip_persistence_actions:
+        if action in SKIP_PERSISTENCE_ACTIONS:
             return False
         return super(ProxyListenerSNS, self).should_persist(method, path, data, headers, response)
 
@@ -407,8 +429,7 @@ async def message_to_subscriber(
     message_attributes = get_message_attributes(req_data)
     if not skip_checks and not check_filter_policy(filter_policy, message_attributes):
         LOG.info(
-            "SNS filter policy %s does not match attributes %s"
-            % (filter_policy, message_attributes)
+            "SNS filter policy %s does not match attributes %s", filter_policy, message_attributes
         )
         return
     if subscriber["Protocol"] == "sms":
@@ -473,13 +494,14 @@ async def message_to_subscriber(
             )
             store_delivery_log(subscriber, True, message, message_id)
         except Exception as exc:
-            LOG.info("Unable to forward SNS message to SQS: %s %s" % (exc, traceback.format_exc()))
+            LOG.info("Unable to forward SNS message to SQS: %s %s", exc, traceback.format_exc())
             store_delivery_log(subscriber, False, message, message_id)
             sns_error_to_dead_letter_queue(subscriber["SubscriptionArn"], req_data, str(exc))
             if "NonExistentQueue" in str(exc):
                 LOG.info(
-                    'Removing non-existent queue "%s" subscribed to topic "%s"'
-                    % (queue_url, topic_arn)
+                    'Removing non-existent queue "%s" subscribed to topic "%s"',
+                    queue_url,
+                    topic_arn,
                 )
                 subscriptions.remove(subscriber)
         return
@@ -518,8 +540,7 @@ async def message_to_subscriber(
                     )
         except Exception as exc:
             LOG.info(
-                "Unable to run Lambda function on SNS message: %s %s"
-                % (exc, traceback.format_exc())
+                "Unable to run Lambda function on SNS message: %s %s", exc, traceback.format_exc()
             )
             store_delivery_log(subscriber, False, message, message_id)
             sns_error_to_dead_letter_queue(subscriber["SubscriptionArn"], req_data, str(exc))
@@ -556,7 +577,7 @@ async def message_to_subscriber(
             response.raise_for_status()
         except Exception as exc:
             LOG.info(
-                "Received error on sending SNS message, putting to DLQ (if configured): %s" % exc
+                "Received error on sending SNS message, putting to DLQ (if configured): %s", exc
             )
             store_delivery_log(subscriber, False, message, message_id)
             sns_error_to_dead_letter_queue(subscriber["SubscriptionArn"], req_data, str(exc))
@@ -569,8 +590,9 @@ async def message_to_subscriber(
             store_delivery_log(subscriber, True, message, message_id)
         except Exception as exc:
             LOG.warning(
-                "Unable to forward SNS message to SNS platform app: %s %s"
-                % (exc, traceback.format_exc())
+                "Unable to forward SNS message to SNS platform app: %s %s",
+                exc,
+                traceback.format_exc(),
             )
             store_delivery_log(subscriber, False, message, message_id)
             sns_error_to_dead_letter_queue(subscriber["SubscriptionArn"], req_data, str(exc))
@@ -592,7 +614,7 @@ async def message_to_subscriber(
             )
             store_delivery_log(subscriber, True, message, message_id)
     else:
-        LOG.warning('Unexpected protocol "%s" for SNS subscription' % subscriber["Protocol"])
+        LOG.warning('Unexpected protocol "%s" for SNS subscription', subscriber["Protocol"])
 
 
 def publish_message(topic_arn, req_data, headers, subscription_arn=None, skip_checks=False):
@@ -607,7 +629,7 @@ def publish_message(topic_arn, req_data, headers, subscription_arn=None, skip_ch
         )
         cache.append(req_data)
 
-    LOG.debug("Publishing message to TopicArn: %s | Message: %s" % (topic_arn, message))
+    LOG.debug("Publishing message to TopicArn: %s | Message: %s", topic_arn, message)
     start_thread(
         lambda _: message_to_subscribers(
             message_id,
